// pm.h - header file for both the game and the level editor
#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <string.h>
#include <allegro.h>

#define SOUND

#ifdef SOUND
// no point in defining DUMB if SOUND is not defined
#define DUMB
#endif

#ifdef DUMB
#include <aldumb.h>
#endif

// these definitions are mutually exclusive
// pick only one and not both!
// FV = full version or MV = my version (debug)
#define FV

// #define PRTSCR
// #define CLONERLINES // does not work since multiple screens
#define KEY_ANIMATION

//#define NETPLAY

//#define DEBUG_FUNCTION_KEYS

// #define FPS_DISPLAY


#define NUM_SPRITES 1024
#define NUM_ANS 256
#define NUM_PLAYERS 4

struct player
   {
     int active;     

     int paused;
     int paused_passcount; // to pause after death, etc...
     int control_method; // 0=normal, 1 = file play, 2 = file record

     float LIFE;
     int LIVES;

     int comp_move;
     int old_comp_move;
     int autoplay[5000][2];
     int api_read;
     int api_write;

     // players position
     float PX, PY;
     int PXint, PYint;

     // background origin in l2000 (maps to SX, SY)
     int WX, WY;

     bool local_draw; // are the values SX,SY,SW,SH valid? ('no' could be remote network player or file play)

     // position and size of players window on screen
     int SX,SY,SW,SH;

     // x move
     float initial_x_speed, max_x_speed;
     float right_speed, left_speed;
     float p_xmove;
     float accel, de_accel;
     int left_right;

     // y move
     float jump_count, fall_count, jump_sinc, fall_sinc;
     float pmovey;
     int ex_jump, player_ride, player_carry;

     // key assignments
     int up_key, down_key, left_key, right_key, jump_key, fire_key, menu_key, map_key, map_held;

     // flags that indicate a control has been activated
     int up, down, left, right, jump, fire, fire_held;

     // bullet stuff
     int bullet_wait_counter, request_bullet, bullet_wait, bullet_speed, num_bullets;

     int num_hits; // the number of enemies this player has hit

   };
extern struct player players[NUM_PLAYERS];


// game global variables


// ----------------game function prototypes----------------------

// all from zplayer.cpp
void player_move();
void draw_player();


void erase_autoplay(int );

int zmenu(int, int, int);
void set_screens(int);
void next_map_mode(void);
void save_apl(void);
void event(int);
void fire_enemy_x_bullet(int, int, int, int);
int is_right_solid(int, int);
int is_left_solid(int, int);
int is_up_solid(int, int, int, int);
int is_down_solid(int, int, int);
void seek_set_xyinc(int EN, int EXint, int EYint);
int get_rot_from_xyinc(int EN);
int find_closest_player(int EXint, int EYint);
void fire_enemy_bulleta(int EN, int EXint, int EYint, int , int p);
int get_rot_from_PXY(int EN, int EXint, int EYint, int p);
int find_closest_player_quad(int EXint, int EYint, int , int prox);
void make_filename(int level_to_load);
void draw_lift_lines(void);
int load_level(int level_to_load, int);
void destroy_lift(int );
void destroy_lift_step(int, int);
int construct_lift(int, char *, int, int, int, int);
int construct_lift_step(int, int, int, int, int, int);
void set_zz(int passcount);
void erase_last_bmsg(void);
void slide_bmsg(void);
void stimp(void);
void maps_and_background_fill(void);
void draw_pop_msg(void);
void draw_bottom_msg(void);
void draw_top_display(void);
void draw_map(void);
void move_lifts(void);
void proc_controllers(void);
void get_new_background(void);
void draw_items(void);
void draw_lifts(void);
void enemy_move(void);
void enemy_draw(void);
void enemy_collision(void);
void proc_lit_rocket(int);
void proc_player_carry(int);
void proc_item_collison(void);
void proc_item_move(void);
void proc_lit_bomb(int);
void stretch_text(char * , char *  , int);
void stretch_text_p(char * , char *, int, int);
void stamp(void);

int load_sprit(void);
void set_map_var(void);
void menu_setup(void);
void set_speed(void);
void set_start_level(int start_level);
void load_keys(void);
int slow_load_and_draw_level(int start_level);
void frame_and_title(void);
void draw_level(void);
void pm_main();
int load_game(void);
void save_game(void);
void help(char *);

void demo_mode(void);

void get_keys(void);

void get_one_key(int p);
void get_all_keys(int p);

int joy_check(int p);

void press_any(void);
int press_any_p_rt(int);

void load_apl(int);
void save_keys(void);
void load_apl(int tf );

void set_controls_from_comp_move(int, int);
void set_comp_move_from_controls(int);

void sc_setup(void);
void add_eilp_to_l2000(void);
void restore_l2000(void);
int item_sort(void);
int sort_enemy(void);
int save_level(int );
void set_lift_initial_not_in_file(int );
void load_sound();
void sound_setup();
void proc_sound();
#ifdef DUMB
void start_music();
#endif

void update_animation(void);
void initialize_zz(void);

// ----------------level editor function prototypes----------------------

int quit_proc();
int load_sprit_proc();
int save_sprit_proc();

int replace_color_proc();
int select_all_proc();
int new_selection_proc();
int gridlines_proc();
int line_draw_proc();
int rect_draw_proc();
int rectfill_draw_proc();
int circle_draw_proc();
int circlefill_draw_proc();
int floodfill_draw_proc();

int rb1_proc(int msg, DIALOG *d, int c);
int rb2_proc(int msg, DIALOG *d, int c);
int rb3_proc(int msg, DIALOG *d, int c);
int rb4_proc(int msg, DIALOG *d, int c);
int rb5_proc(int msg, DIALOG *d, int c);
int rb6_proc(int msg, DIALOG *d, int c);
int rb7_proc(int msg, DIALOG *d, int c);
int ar1_proc(int msg, DIALOG *d, int c);
int ar2_proc(int msg, DIALOG *d, int c);
int ar3_proc(int msg, DIALOG *d, int c);
int ar4_proc(int msg, DIALOG *d, int c);
int ar5_proc(int msg, DIALOG *d, int c);
int ar6_proc(int msg, DIALOG *d, int c);
int bm_rot_proc();
int bm_flip_x_axis_proc();
int bm_flip_y_axis_proc();
int bm_scroll_up_proc();
int bm_scroll_down_proc();
int bm_scroll_left_proc();
int bm_scroll_right_proc();

// all from e_fnx.cpp
void update_animation(void);
void initialize_zz(void);
void set_rocket_rot(int num, int x2, int y2);
void set_xyinc_rot(int EN, int x2, int y2);
void set_wx(int x, int y);
void set_wx_from_start_block(void);
int get_rot_from_xyinc(int EN);
void show_big(void);
void draw_big(void);
void draw_bs(int cc);
int getbox(char *txt);
int getxy(char *txt, int obj_type, int sub_type, int num );
int edit_float(int x, int y);
int edit_int(int x, int y, int initial, int inc, int lv, int uv);
void set_swbl(int start_shape, int end_shape, int mt);
int draw_framed_shape(int x, int y, int shape, int frame_width, int ci);


// all from e_bitmap.cpp

int draw_current_bitmap(int msg, DIALOG *d, int c);
int new_draw_csp(int msg, DIALOG *d, int c);
int show_draw_mode(int msg, DIALOG *d, int c);
int draw_animation(int msg, DIALOG *d, int c);
int color_bars(int msg, DIALOG *d, int c);
int fade_proc(int msg, DIALOG *d, int c);
int select_bitmap_ans();
int animation_proc();
int select_bitmap_proc();
int copy_bitmap_proc();
void bitmap_main(void);
int edit_pallete_proc();




// all from e_editor.cpp
void tsw(void);
void draw_enemy_shape_l2000(int e);
void draw_item_shape(int num, int x, int y);
void draw_enemy_shape(int e, int x, int y);
void show_draw_item_cursor(void);
void set_block_range(void);
void update(void);
int get_new_box(void);
int process_scrolledge(void);
void pointer_text(int x1, int y1, int tx, int ty);
void do_brf(int x, int y);
void look_up(int tlc, int rb);
void look_down(int tlc, int rb);
int zoom_full_screen(int wx, int wy, int draw_item);
void draw_item_info(int x, int y, int color, int type, int num);
void edit_menu(int el);

// all from e_glt.cpp
void global_level();

// all from e_guifnx.cpp
int quit_proc();
int load_sprit_proc();
int save_sprit_proc();
int replace_color_proc();
int gridlines_proc();
int line_draw_proc();
int rect_draw_proc();
int rectfill_draw_proc();
int new_selection();
int select_all();
int rectfill_draw_proc();
int circle_draw_proc();
int circlefill_draw_proc();
int floodfill_draw_proc();

int rb1_proc(int msg, DIALOG *d, int c);
int rb2_proc(int msg, DIALOG *d, int c);
int rb3_proc(int msg, DIALOG *d, int c);
int rb4_proc(int msg, DIALOG *d, int c);
int rb5_proc(int msg, DIALOG *d, int c);
int rb6_proc(int msg, DIALOG *d, int c);
int rb7_proc(int msg, DIALOG *d, int c);
int ar1_proc(int msg, DIALOG *d, int c);
int ar2_proc(int msg, DIALOG *d, int c);
int ar3_proc(int msg, DIALOG *d, int c);
int ar4_proc(int msg, DIALOG *d, int c);
int ar5_proc(int msg, DIALOG *d, int c);
int ar6_proc(int msg, DIALOG *d, int c);
int bm_rot_proc();
int bm_flip_x_axis_proc();
int bm_flip_y_axis_proc();
int bm_scroll_up_proc();
int bm_scroll_down_proc();
int bm_scroll_left_proc();
int bm_scroll_right_proc();

// all from e_item.cpp

void draw_edit_item_shape(int x, int y, int obt, int sub_type, int num);
void edit_cb(int mb, int num);
void edit_hd(int mb, int num);
void draw_button(int b_num, char *msg, int tc, int rc);
void crosshairs(int mx, int my, int x, int y, int color); /* funtion to draw rentangle and crosshairs */
void title(char *txt, int y, int tc, int fc);
int item_sort(void);
void title_obj(int obj_type, int sub_type, int num);
void edit_item(int ei, int num);


// all from e_main.cpp

void final_wrapup(void);
int initial_setup(void);
int main(int argument_count, char **argument_array);

// all from e_nev.cpp
void draw_pmsg(int c, int x, int y);
void erase_item(int num);
int get_empty_item(int type); /* finds, sets, sorts refinds */
int get_empty_enemy(int type); /* type */
void recalc_pod(int num);
int move_trigger_box(int num);
int move_pod_extended(int num);
int sort_enemy(void);
void create_cloner(void);
int create_pod(void);
int msg_text_editor(int msg);
void show_cursor(char *f, int cursor_pos, int cursor_color);
void dpm(int c, char *f, int cursor_pos);
int create_pmsg(int c, int new_message);
int create_obj(int obt, int sub_type, int sent_num);
int create_item(int type);

// all from e_pde.cpp

int load_PDE();
int bottom_menu(int menu_num);
void save_PDE();
void predefined_enemies(void);

// all from e_sel.cpp

int load_selection(void);
int save_selection(void);
void do_copy(int qx1, int qy1);
void do_fcopy(int qx1, int qy1);
void do_clear(void);
void draw_fsel(void);

// all from e_sliders.cpp

void update_var(int bn, int num, float f);

void fill_smsg(int bn, int num);

float draw_slider(float sdx, float sul, float sll,int x1, int y1, int x2, int y2, int dm, int col);

void color_selector(int x1, int y1, int x2, int y2, int bn, int num,
              int type, int obt, int q0, int q1, int q2, int q3, int q4, int q5, int q6, int q7 );

int new_button(int x1, int y1, int x2, int y2, int bn, int num,
                int type, int obt, int q0, int q1, int q2, int q3, int q4, int q5, int q6, int q7 );

int edit_int_slider(int x1, int y1, int x2, int y2, int bn, int num,
                 int type, int obt, int q0, int q1, int q2, int q3, int q4, int q5, int q6, int q7 );

// all from e_special.cpp

void draw_status_window(void); /* only at setup */
int process_status_window(void);
void draw_select_window(void);
int process_select_window();

// all from yfilecom.cpp
void smake_filename(int x);
void make_filename(int x);
int load();
int save();
int load_sprit(void);
int load_level(int level_to_load, int display);
int save_level(int level_to_save);

#ifdef MV
int save_sprit(void);
#endif

#ifdef OLDFNXSTORAGE
int game_load_level(int level_to_load, int display);
#endif

// from bled

int block_editor_proc();
int select_block_proc();

// all from e_nlv.cpp



void show_all_lifts(void);
int construct_lift(int l,char* lift_name,int width,int height,int color,int num_steps);
void destroy_lift(int lift);
int construct_lift_step(int lift, int step, int x, int y, int val, int type);
void destroy_lift_step(int lift, int step);
void erase_lift(int lift);
void set_lift_xyinc(int d, int step); /* used by move lift everywhere */
void draw_steps(int highlight_step, int fc);
void fast_draw_lifts(void);
void draw_lift_mp(); /* draws the current lift on mp */
int get_new_lift_step(int step);
void draw_step_button(int txc, int ty, int lift, int step, int tc, int rc);
int create_lift(void);
void draw_big_no_lifts(void);
void lift_setup(void);
void set_lift_initial_not_in_file(int lift);
void set_lift(int lift, int step);
void move_lifts(int times);
void fill_ll(void);
void lift_editor(int lift);


// from e_menu.cpp
int pmenu(int menu_num);
void text_setup();

















